"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformExternalModule = void 0;
require("source-map-support/register");
const path_1 = require("path");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const fancy_log_1 = require("fancy-log");
const typescript_1 = require("typescript");
exports.default = typescript_transformer_common_1.createProgramPlugin(function plugin(program, { verbose } = {}, { packageJsonPath }) {
    const debug = typescript_transformer_common_1.getDebug(verbose || false);
    debug('load package.json: %s', packageJsonPath);
    const dependencies = typescript_transformer_common_1.createDependencies(packageJsonPath);
    debug('    dependencies count: %s', Object.keys(dependencies).length);
    return function transformer(transformationContext) {
        transformationContext.enableEmitNotification(typescript_1.SyntaxKind.SourceFile);
        function visitNode(node) {
            const fileName = node.getSourceFile().fileName;
            if (typescript_transformer_common_1.isImportExport(node)) {
                const importId = typescript_transformer_common_1.getImportedName(node);
                if (typescript_transformer_common_1.isImportNodeBuiltins(importId)) {
                    return node;
                }
                debug(' * %s', node.getText(node.getSourceFile()).split('\n').join(' '));
                if (typescript_transformer_common_1.isImportFromNodeModules(importId)) {
                    const { packageName } = typescript_transformer_common_1.splitPackageName(importId);
                    const info = typescript_transformer_common_1.resolveTypescriptModule(node, packageJsonPath);
                    if (info.type === 'missing') {
                        fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
                    }
                    else if (info.specifier.length > 0) {
                        if (!dependencies[packageName]) {
                            fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
                        }
                        debug('   -> replace to %s', info.nodeResolve);
                        return typescript_transformer_common_1.replaceImportExportSpecifier(node, info.nodeResolve);
                    }
                }
                else {
                    debug('   -> not imported');
                    // not node_module
                }
                if (typescript_transformer_common_1.testProjectFile(debug, fileName, node, program)) {
                    const importPath = node.moduleSpecifier.text;
                    return typescript_transformer_common_1.replaceImportExportSpecifier(node, importPath + '.js');
                }
            }
            return node;
        }
        return (sourceFile) => {
            const { fileName } = sourceFile;
            debug('[trans] visit \x1B[2m%s\x1B[0m', fileName);
            const result = typescript_1.visitEachChild(sourceFile, visitNode, transformationContext);
            debug('[trans] visit complete');
            return result;
        };
    };
});
function transformExternalModule(debug, node, packageJsonPath) {
    const importPath = node.moduleSpecifier.text;
    const { packageName, filePath } = typescript_transformer_common_1.splitPackageName(importPath);
    const result = typescript_transformer_common_1.resolveModule('module', path_1.resolve(packageJsonPath, '../node_modules', packageName, 'package.json'), filePath, true);
    if (result) {
        debug('resolve module result:', result);
        const id = path_1.relative(path_1.resolve(packageJsonPath, '../node_modules'), result).replace(/^\.\//, '');
        return typescript_transformer_common_1.replaceImportExportSpecifier(node, id);
    }
    else {
        fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
    }
    return node;
}
exports.transformExternalModule = transformExternalModule;
//# sourceMappingURL=transform.js.map